#!/usr/bin/perl

# $Id: find-lang.pl 224280 2007-07-05 22:27:39Z nanardon $

use strict;
use warnings;
use File::Find;
use Getopt::Long;
use Pod::Usage;

GetOptions(
    'all-name'   => \my $allname,
    'with-gnome' => \my $withgnome,
    'with-kde'   => \my $withkde,
    'with-html'  => \my $withhtml,
    'without-mo' => \my $withoutmo,
    'with-man' => \my $withman,
) or pod2usage();

my ($buildroot, @searchname) = @ARGV;
$buildroot =~ s:/+$::; # removing trailing /
my ($pkgname) = @searchname or die "Main name to find missing\n";
$buildroot or die "No buildroot given\n";

my %finallist; # filename => attr, easy way to perform uniq

File::Find::find(
    sub {
        my $file = substr($File::Find::name, length($buildroot));
        my ($pkg, $lang, $parent);
        if ($file =~ m:^((.*/share/locale)/([^/_@]+)[^/]*).*/([^/]+)\.mo:) {
            if ($withoutmo) {
                return;
            }
            ($pkg, $lang, $parent) = ($4, $3, $2);
        } elsif ($file =~ m:^(.*/gnome/help/([^/_@]+))[^/]*/([^/]+)$:) {
            if (!$withgnome) {
                return;
            }
            ($pkg, $lang, $parent) = ($2, $3, $1);
        } elsif ($file =~ m:^(.*/doc/kde/HTML)/([^/_@]+)[^/]*/([^/]+)$:) {
            if (!$withkde) {
                return;
            }
            ($pkg, $lang, $parent) = ($2, $3, $1);
        } elsif ($file =~ m:^(.*/doc/HTML)/([^/_@]+)[^/]*/([^/]+)$:) {
            if (!$withhtml) {
                return;
            }
            ($pkg, $lang, $parent) = ($3, $2, $1);
        } elsif ($file =~ m:^(/+usr/share/man)/([^/_@\.]+)[^/]*/man[^/]+/([^/.]+)\.\d[^/]*$:) {
            if (!$withman) {
                return;
            }
            ($pkg, $lang, $parent) = ($3, $2, $1);
            $file .= '*';
        } else {
            return;
        }
        if (! ((grep { $_ eq $pkg } @searchname) || $allname)) {
            return;
        }
        my $rpmlang = $lang eq 'C' ? "" : "%lang($lang)";

        parent_to_own($parent, $file, $rpmlang);
        $finallist{$file} = "$rpmlang ";

    },
    $buildroot || '/'
);

open(my $hlang, '>', "$pkgname.lang") or die "canno't open $pkgname.lang\n";

foreach (sort keys %finallist) {
    print $hlang "$finallist{$_}$_\n";
}

close($hlang);

exit(0);

sub parent_to_own {
    my ($parent, $file, $lang) = @_;
    my @subdir = grep { $_ } split('/', substr($file, length($parent)));
    pop(@subdir);
    #$finallist{$parent} = "$lang %dir";
    while (my $part = shift(@subdir)) {
        $parent .= "/$part";
        $finallist{$parent} = "$lang %dir ";
    }
}
